package common.world;

import java.util.Collection;
import java.util.List;
import java.util.function.Predicate;

import common.biome.Biome;
import common.block.Block;
import common.block.LeavesType;
import common.entity.Entity;
import common.entity.item.EntityExplosion;
import common.entity.npc.EntityNPC;
import common.init.SoundEvent;
import common.item.ItemStack;
import common.model.ParticleType;
import common.tileentity.TileEntity;
import common.util.BlockPos;
import common.util.BoundingBox;
import common.util.Facing;
import common.util.HitPosition;
import common.util.Vec3;

public interface IWorld {

	boolean isBlockSolid(BlockPos pos);

	void setGravity(float gravity);

	Biome getBiomeGenForCoords(BlockPos pos);

	boolean isAirBlock(BlockPos pos);

	boolean isBlockLoaded(BlockPos pos);

	boolean isBlockLoaded(BlockPos pos, boolean allowEmpty);

	boolean isAreaLoaded(BlockPos center, int radius);

	boolean isAreaLoaded(BlockPos center, int radius, boolean allowEmpty);

	boolean isAreaLoaded(BlockPos from, BlockPos to);

	boolean isAreaLoaded(BlockPos from, BlockPos to, boolean allowEmpty);

	Chunk getChunk(BlockPos pos);

	boolean setState(BlockPos pos, State newState, int flags);

	boolean setState(BlockPos pos, State state);

	boolean setBlockToAir(BlockPos pos);

	boolean destroyBlock(BlockPos pos, boolean dropBlock);

	void notifyNeighborsRespectDebug(BlockPos pos, Block blockType);

	void markBlocksDirtyVertical(int x1, int z1, int x2, int z2);

	void markBlockRangeForRenderUpdate(BlockPos rangeMin, BlockPos rangeMax);

	void notifyNeighborsOfStateChange(BlockPos pos, Block blockType);

	void notifyNeighborsOfStateExcept(BlockPos pos, Block blockType, Facing skipSide);

	void notifyBlockOfStateChange(BlockPos pos, Block blockIn);

	boolean canSeeSky(BlockPos pos);

	int getLight(BlockPos pos);

	int getLightFromNeighbors(BlockPos pos);

	BlockPos getHeight(BlockPos pos);

	int getChunksLowestHorizon(int x, int z);

	int getLightFromNeighborsFor(LightType type, BlockPos pos);

	int getLightFor(LightType type, BlockPos pos);

	void setLightFor(LightType type, BlockPos pos, int lightValue);

	int getCombinedLight(BlockPos pos, int lightValue);

	float getLightBrightness(BlockPos pos);

	State getState(BlockPos pos);

	BlockPos getBlockTrace(Entity entity, int distance);

	HitPosition rayTraceBlocks(Vec3 p_72933_1_, Vec3 p_72933_2_);

	HitPosition rayTraceBlocks(Vec3 start, Vec3 end, boolean stopOnLiquid);

	HitPosition rayTraceBlocks(Vec3 vec31, Vec3 vec32, boolean stopOnLiquid, boolean ignoreBlockWithoutBoundingBox,
			boolean returnLastUncollidableBlock);

	void playSoundAtEntity(Entity entityIn, SoundEvent name, float volume);

	boolean spawnEntityInWorld(Entity entityIn);

	void removeEntity(Entity entityIn);

	List<BoundingBox> getCollidingBoundingBoxes(Entity entityIn, BoundingBox bb);

	List<BoundingBox> getCollisionBoxes(BoundingBox bb);

	int calculateSkylightSubtracted(boolean current);

	float getCelestialAngle(float partialTicks);

	int getMoonPhase();

	float getCurrentMoonPhaseFactor();

	float getCelestialAngleRadians(float partialTicks);

	BlockPos getPrecipitationHeight(BlockPos pos);

	void updateEntities();

	boolean addTileEntity(TileEntity tile);

	void addTileEntities(Collection<TileEntity> tileEntityCollection);

	void updateEntity(Entity entityIn, boolean forceUpdate);

	boolean checkNoEntityCollision(BoundingBox bb);

	boolean checkNoEntityCollision(BoundingBox bb, Entity entityIn);

	boolean isAnyLiquid(BoundingBox bb);

	boolean isFlammableWithin(BoundingBox bb);

	boolean handleLiquidAcceleration(BoundingBox bb, Entity entityIn);

	boolean isMaterialInMolten(BoundingBox bb);

	boolean isAABBInLiquid(BoundingBox bb);

	Explosion createExplosion(Entity entityIn, double x, double y, double z, float strength, boolean isSmoking);

	Explosion createAltExplosion(Entity entityIn, double x, double y, double z, float strength, boolean isSmoking);

	Explosion newExplosion(Entity entityIn, double x, double y, double z, float strength, boolean isFlaming, boolean isSmoking, boolean altSound);

	EntityExplosion newExplosion(double x, double y, double z, int strength);

	float getBlockDensity(Vec3 vec, BoundingBox bb);

	boolean extinguishFire(EntityNPC player, BlockPos pos, Facing side);

	TileEntity getTileEntity(BlockPos pos);

	void setTileEntity(BlockPos pos, TileEntity tileEntityIn);

	void removeTileEntity(BlockPos pos);

	void markTileEntityForRemoval(TileEntity tileEntityIn);

	float getTempOffset();

	LeavesType getLeavesGen(BlockPos pos);

	float getTemperatureK(BlockPos pos);

	float getTemperatureC(BlockPos pos);

	boolean canFreezeAt(BlockPos pos);

	boolean canBurnAt(BlockPos pos);

	boolean doesWaterVaporize(BlockPos pos);

	boolean isLavaFaster(BlockPos pos);

	boolean canSnowAt(BlockPos pos, boolean checkLight, boolean allowLayers);

	boolean checkLight(BlockPos pos);

	boolean checkLightFor(LightType lightType, BlockPos pos);

	List<Entity> getEntitiesWithinAABBExcludingEntity(Entity entityIn, BoundingBox bb);

	List<Entity> getEntitiesInAABBexcluding(Entity entityIn, BoundingBox boundingBox, Predicate<? super Entity> predicate);

	<T extends Entity> List<T> getEntitiesWithinAABB(Class<? extends T> classEntity, BoundingBox bb);

	<T extends Entity> List<T> getEntitiesWithinAABB(Class<? extends T> clazz, BoundingBox aabb, Predicate<? super T> filter);

	Entity getEntityByID(int id);

	void markChunkDirty(BlockPos pos, TileEntity unusedTileEntity);

	void loadEntities(Collection<Entity> entityCollection);

	void unloadEntities(Collection<Entity> entityCollection);

	boolean canBlockBePlaced(Block blockIn, BlockPos pos, boolean p_175716_3_, Facing side, Entity entityIn, ItemStack itemStackIn);

	int getSeaLevel();

	int getStrongPower(BlockPos pos);

	boolean isSidePowered(BlockPos pos, Facing side);

	int getRedstonePower(BlockPos pos, Facing facing);

	boolean isBlockPowered(BlockPos pos);

	int isBlockIndirectlyGettingPowered(BlockPos pos);

	EntityNPC getClosestPlayerToEntity(Entity entityIn, double distance);

	EntityNPC getClosestPlayer(double x, double y, double z, double distance);

	boolean isAnyPlayerWithinRangeAt(double x, double y, double z, double range);

	void setTimeFactor(int factor);

	void setEntityState(Entity entityIn, byte state);

	void addBlockEvent(BlockPos pos, Block blockIn, int eventID, int eventParam);

	Weather getWeather();

	long getDayTime();

	void setWeather(Weather weather);

	void setDayTime(long time);

	float getDarkness();

	void setDarkness(float dark);

	float getRainStrength();

	void setRainStrength(float strength);

	float getFogStrength();

	void setFogStrength(float strength);

	void setTemperature(float temp);

	boolean isDark();

	boolean isThundering();

	boolean hasDownfall();

	boolean isRaining();

	boolean isRainingAt(BlockPos strikePosition, boolean wet);

	void playAuxSFX(int type, BlockPos pos, int data);

	void updateComparatorOutputLevel(BlockPos pos, Block blockIn);

	void scheduleUpdate(BlockPos pos, Block blockIn, int delay);

	void spawnParticle(ParticleType particleType, double xCoord, double yCoord, double zCoord, double xOffset, double yOffset, double zOffset,
			int... data);

	Chunk getChunk(int x, int z);

	void markBlockForUpdate(BlockPos pos);

	void markBlockRangeForRenderUpdate(int x1, int y1, int z1, int x2, int y2, int z2);

	void playSound(SoundEvent sound, double x, double y, double z, float volume);

	void playAuxSFX(EntityNPC player, int sfxType, BlockPos blockPosIn, int data);

	void sendBlockBreakProgress(int breakerId, BlockPos pos, int progress);

}